#include "Gyro.h"

bool Gyro::Gyro_First_Loop_Done = false;

Axes_long Gyro::Acc_Raw = {0,0,0};
int Gyro::Temp = 0;
Axes_long Gyro::Gyro_Raw = {0,0,0};

Axes Gyro::Gyro_value = {0,0,0};
Axes Gyro::Acc_value = {0,0,0};

Axes Gyro::Gyro_Calc = {0,0,0};

void Gyro::GY_init() {
    Wire.beginTransmission(GYRO_ADDRESS);
    Wire.write(0x6B);
    Wire.write(0);
    Wire.endTransmission(true);
}

void Gyro::GY_Read() {
    Wire.beginTransmission(GYRO_ADDRESS);
    Wire.write(0x3B);
    Wire.endTransmission(false);
    Wire.requestFrom(GYRO_ADDRESS,14,true);
    byte Acc_X_msb = Wire.read();
    byte Acc_X_lsb = Wire.read();
    byte Acc_Y_msb = Wire.read();
    byte Acc_Y_lsb = Wire.read();
    byte Acc_Z_msb = Wire.read();
    byte Acc_Z_lsb = Wire.read();
    byte Temp_msb = Wire.read();
    byte Temp_lsb = Wire.read();
    byte Gyro_X_msb = Wire.read();
    byte Gyro_X_lsb = Wire.read();
    byte Gyro_Y_msb = Wire.read();
    byte Gyro_Y_lsb = Wire.read();
    byte Gyro_Z_msb = Wire.read();
    byte Gyro_Z_lsb = Wire.read();

    Acc_Raw.x = (Acc_X_msb << 8) | Acc_X_lsb;
    Acc_Raw.y = (Acc_Y_msb << 8) | Acc_Y_lsb;
    Acc_Raw.z = (Acc_Z_msb << 8) | Acc_Z_lsb;
    Temp = (Temp_msb << 8) | Temp_lsb;
    Gyro_Raw.x = (Gyro_X_msb << 8) | Gyro_X_lsb;
    Gyro_Raw.y = (Gyro_Y_msb << 8) | Gyro_Y_lsb;
    Gyro_Raw.z = (Gyro_Z_msb << 8) | Gyro_Z_lsb;
    
    Gyro_Raw.x -= GYRO_CALLIBRATION_CORRECTION_X;
    Gyro_Raw.y -= GYRO_CALLIBRATION_CORRECTION_Y;
    Gyro_Raw.z -= GYRO_CALLIBRATION_CORRECTION_Z;

    Acc_Raw.x -= ACC_CALLIBRATION_CORRECTION_X;
    Acc_Raw.y -= ACC_CALLIBRATION_CORRECTION_Y;
    Acc_Raw.z -= ACC_CALLIBRATION_CORRECTION_Z;
}

Axes Gyro::GY_Get_Axes() {  
    GY_Read();
    
    Gyro_Calc.x += Gyro_Raw.x * (1/(GYRO_REFRESH_RATE*GYRO_SENSITIVITY));
    Gyro_Calc.y += Gyro_Raw.y * (1/(GYRO_REFRESH_RATE*GYRO_SENSITIVITY));
    Gyro_Calc.z = Gyro_Raw.z * (1/(GYRO_REFRESH_RATE*GYRO_SENSITIVITY));
    
    Gyro_Calc.x += Gyro_Calc.y * sin(Gyro_Calc.z * DEG_TO_RAD);
    Gyro_Calc.y -= Gyro_Calc.x * sin(Gyro_Calc.z * DEG_TO_RAD);
    
    Acc_value.z = sqrt((Acc_Raw.x*Acc_Raw.x) + (Acc_Raw.y*Acc_Raw.y) + (Acc_Raw.z*Acc_Raw.z));
    Acc_value.x = asin((float)Acc_Raw.y/Acc_value.z) * RAD_TO_DEG;
    Acc_value.y = asin((float)Acc_Raw.x/Acc_value.z) * -RAD_TO_DEG;
    
    if(Gyro_First_Loop_Done == true) {
      Gyro_Calc.x = Gyro_Calc.x * (1 - GYRO_GYRO_CORRECTION) + Acc_value.x * GYRO_GYRO_CORRECTION;
      Gyro_Calc.y = Gyro_Calc.y * (1 - GYRO_GYRO_CORRECTION) + Acc_value.y * GYRO_GYRO_CORRECTION;
    } else {
      Gyro_Calc.x = Acc_value.x;
      Gyro_Calc.y = Acc_value.y;
      Gyro_First_Loop_Done = true;
    }
    
    Gyro_value.x = Gyro_Calc.x;
    Gyro_value.y = Gyro_Calc.y;

    return {Gyro_value.x, Gyro_value.y, Gyro_value.z};
}
