#ifndef GYRO
#define GYRO

#include <Wire.h>
#include <SPI.h>

#define DEG_TO_RAD (PI/180)
#define RAD_TO_DEG (180/PI)

#define GYRO_ADDRESS 0x68
#define GYRO_REGISTER_ACC_X_MSB 0x3B
#define GYRO_REGISTER_ACC_X_LSB 0x3C
#define GYRO_REGISTER_ACC_Y_MSB 0x3D
#define GYRO_REGISTER_ACC_Y_LSB 0x3E
#define GYRO_REGISTER_ACC_Z_MSB 0x3F
#define GYRO_REGISTER_ACC_Z_LSB 0x40
#define GYRO_REGISTER_TEMP_MSB 0x41
#define GYRO_REGISTER_TEMP_LSB 0x42
#define GYRO_REGISTER_GYRO_X_MSB 0x43
#define GYRO_REGISTER_GYRO_X_LSB 0x44
#define GYRO_REGISTER_GYRO_Y_MSB 0x45
#define GYRO_REGISTER_GYRO_Y_LSB 0x46
#define GYRO_REGISTER_GYRO_Z_MSB 0x47
#define GYRO_REGISTER_GYRO_Z_LSB 0x48
#define GYRO_REFRESH_RATE 200
#define GYRO_SENSITIVITY 65.5
#define GYRO_GYRO_CORRECTION 0.04

#define GYRO_CALLIBRATION_CORRECTION_X 168
#define GYRO_CALLIBRATION_CORRECTION_Y -50
#define GYRO_CALLIBRATION_CORRECTION_Z 103

#define ACC_CALLIBRATION_CORRECTION_X 1580
#define ACC_CALLIBRATION_CORRECTION_Y -566
#define ACC_CALLIBRATION_CORRECTION_Z 14753

#ifndef STRUCTS
#define STRUCTS

struct Axes {
    float x;
    float y;
    float z;
};

struct Axes_long {
    long x;
    long y;
    long z;
};

struct RF_data {
  byte thrust;
  byte x;
  byte y;
  byte rot;
  byte data;
};

#endif 

class Gyro 
{

public:
    //FUNCTIONS
    void GY_init();
    Axes GY_Get_Axes();

    //VARIABLES
    //Raw Data
    static Axes_long Gyro_Raw;

    //Data
    static Axes Gyro_value;
    
    
private:
    //FUNCTIONS
    void GY_Read();

    //VARIABLES
    //Raw Data
    static int Temp;
    static Axes_long Acc_Raw;
    
    //Data
    static Axes Acc_value;
    
    //Calculations
    static Axes Gyro_Calc;
    static bool Gyro_First_Loop_Done;

};

#endif
