#include "Mag.h"

Axes Mag::MG_value = {0,0,0};
int Mag::Mag_Callibrate = 0;

void Mag::MG_init() {
  MG_Write(MG_REGISTER_MODE, 0x00);
  MG_Set_Gain(1.3);
}

void Mag::MG_Callibrate() {
  long z_value_total = 0;
  for(int i = 0; i < 2000; i++){
    z_value_total += MG_Get_Data().z;
  }
  Mag_Callibrate = z_value_total / 2000;
}

void Mag::MG_Write(byte reg, byte value) {
  Wire.beginTransmission(MG_DEVICE_ADDRESS);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission();
}

byte Mag::MG_Read(byte reg) {
  Wire.beginTransmission(MG_DEVICE_ADDRESS);
  Wire.write(reg);
  Wire.endTransmission();
  Wire.requestFrom(MG_DEVICE_ADDRESS, 1);
  byte value = Wire.read();
  Wire.endTransmission();
  return value;
}

void Mag::MG_Set_Gain(float gain) {
  byte value = 0x20;
  if(gain == 0.88){
    value = 0x00;
  } else if(gain == 1.3) {
    value = 0x20;
  } else if(gain == 1.9) {
    value = 0x40;
  } else if(gain == 2.5) {
    value = 0x60;
  } else if(gain == 4.0) {
    value = 0x80;
  } else if(gain == 4.7) {
    value = 0xA0;
  } else if(gain == 5.6) {
    value = 0xC0;
  } else if(gain == 8.1) {
    value = 0xE0;
  }
  
  MG_Write(MG_REGISTER_B, value);
  
}

Axes_long Mag::MG_Get_Data() {
  Wire.beginTransmission(MG_DEVICE_ADDRESS);
  Wire.write(MG_REGISTER_X_MSB);
  Wire.endTransmission();
  Wire.requestFrom(MG_DEVICE_ADDRESS, 6);
  while (Wire.available() < 6);
  byte x_msb = Wire.read();
  byte x_lsb = Wire.read();
  byte z_msb = Wire.read();
  byte z_lsb = Wire.read();
  byte y_msb = Wire.read();
  byte y_lsb = Wire.read();

  int x_reg_value = ((x_msb << 8) | x_lsb);
  int z_reg_value = ((z_msb << 8) | z_lsb);
  int y_reg_value = ((y_msb << 8) | y_lsb);
  
  return {x_reg_value, y_reg_value, z_reg_value};
}


//************
//  GET DATA
//************

float Mag::MG_Get_Direction(Axes Gyro_value) {
  
  Axes_long reg_value = MG_Get_Data();

  reg_value.z -= Mag_Callibrate;

  float x_value = reg_value.x / MG_GAUSS_XY * GAUSS_TO_MICROTESLA;
  float y_value = reg_value.y / MG_GAUSS_XY * GAUSS_TO_MICROTESLA;
  float z_value = reg_value.z / MG_GAUSS_Z * GAUSS_TO_MICROTESLA;

  z_value = -z_value;
  y_value = -y_value;
  
  //float Mag_roll = Gyro_value.x * DEG_TO_RAD;
  //float Mag_pitch = -Gyro_value.y * DEG_TO_RAD;

  float Mag_roll = -Gyro_value.x * DEG_TO_RAD;
  float Mag_pitch = -Gyro_value.y * DEG_TO_RAD;
  
  Serial.print(Mag_roll);
  Serial.print("  ");
  Serial.print(Mag_pitch);
 
  Serial.print(" ... ");
  
  Serial.print(x_value);
  Serial.print("  ");
  Serial.print(y_value);
  Serial.print("  ");
  Serial.print(z_value);
  Serial.print("  ");
  
  x_value = x_value * cos(Mag_pitch) + y_value * sin(Mag_roll) * sin(Mag_pitch) + z_value * cos(Mag_roll) * sin(Mag_pitch); 
  y_value = y_value * cos(Mag_roll) - z_value * sin(Mag_roll);

  /*Serial.print(x_value);
  Serial.print("  ");
  Serial.print(y_value);
  Serial.print("  ");
  Serial.println(z_value);*/
  
  /*Serial.print(x_value);
  Serial.print("  ");
  Serial.print(y_value);
  Serial.println("  ");*/
  
  MG_value.x = MG_value.x * 0.9 + x_value * 0.1;
  MG_value.y = MG_value.y * 0.9 + y_value * 0.1;
  
  float heading = atan2(MG_value.y,MG_value.x);
  heading += MY_DECLANATION_ANGLE;

  if(heading < 0){
    heading += 2*PI;
  }

  if(heading > 2*PI){
    heading -= 2*PI;
  }
  Serial.println(heading * RAD_TO_DEG);
  return heading * RAD_TO_DEG;
  
}
