#include "Radio.h"

//***************
//RF INITIALISE
//***************
void Radio::RF_init() {
  pinMode (RF_CSN, OUTPUT);
  SPI.begin();
  //SPI.beginTransaction(SPISettings(1000000, MSBFIRST, SPI_MODE0));
  delay(5);
  RF_Register_Write(RF_REGISTER_CONFIG, 0x0C); //INTERRUPT NOT REFLECTED ON THE IRQ PIN, ENABLE CRC ENCODING SCHEME, ENABLE 2-BYTES CRCO
  RF_Register_Write(RF_REGISTER_SETUP_RETR, 0x5F); //SET RETRANSMIT DELAY TO 1500uS and up to 15 Retransmits
  RF_Set_Data_Rate(1000);
  //toggle_features();
  RF_Register_Write(RF_REGISTER_FEATURE, 0x00);
  RF_Register_Write(RF_REGISTER_DYNPD, 0x00);
  RF_Register_Write(RF_REGISTER_STATUS, 0x00);
  RF_Set_Channel(76);
  RF_Flush_Rx();
  RF_Flush_Tx();
  RF_Power_Enable(true);
  RF_Register_Write(RF_REGISTER_CONFIG, (RF_Register_Read(RF_REGISTER_CONFIG) & 0xFE));
}

//*****************************************
//  RF WRITING AND READING FROM REGISTERS
//*****************************************
void Radio::RF_Register_Write(byte reg, byte value) {
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_W_REGISTER | (RF_MASK_WRITE & reg));
  SPI.transfer(value);
  digitalWrite (RF_CSN, HIGH);
}

void Radio::RF_Register_Write_Mult(byte reg, byte value[5]) {
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_W_REGISTER | (RF_MASK_WRITE & reg));
  SPI.transfer(value[0]);
  SPI.transfer(value[1]);
  SPI.transfer(value[2]);
  SPI.transfer(value[3]);
  SPI.transfer(value[4]);
  digitalWrite (RF_CSN, HIGH);
}

byte Radio::RF_Register_Read(byte reg) {
  digitalWrite (RF_CSN, LOW);
  byte RF_Status = SPI.transfer(RF_WORD_R_REGISTER | (RF_MASK_READ & reg));
  byte RF_Data = SPI.transfer(0x00);
  digitalWrite (RF_CSN, HIGH);
  return RF_Data;
}

//*********************
//  SETTING FUNCTIONS
//*********************
void Radio::RF_Set_Data_Rate(int rate) {
  if(rate == 250) {
    byte setup_read = RF_Register_Read(RF_REGISTER_RF_SETUP);
    RF_Register_Write(RF_REGISTER_RF_SETUP,((setup_read & 0xD7) | 0x20));
  } else if (rate == 1000) {
    byte setup_read = RF_Register_Read(RF_REGISTER_RF_SETUP);
    RF_Register_Write(RF_REGISTER_RF_SETUP,((setup_read & 0xD7) | 0x00));
  } else if (rate == 2000) {
    byte setup_read = RF_Register_Read(RF_REGISTER_RF_SETUP);
    RF_Register_Write(RF_REGISTER_RF_SETUP,((setup_read & 0xD7) | 0x01));
  }
}

void Radio::RF_Set_Channel(byte channel) {
  RF_Register_Write(RF_REGISTER_RF_CH,channel);
}

void Radio::RF_Set_Reading_Pipe(int pipe, bool enable, byte address[5]){
  byte pipe_write = pow(2,pipe);

  if(enable == true){
    if(pipe < 6 && pipe >= 0) {
      if(pipe == 0){
        RF_Register_Write_Mult(RF_REGISTER_RX_ADDR_P0, address);
      } else if(pipe == 1){
        RF_Register_Write_Mult(RF_REGISTER_RX_ADDR_P1, address);
      } else if(pipe == 2){
        RF_Register_Write(RF_REGISTER_RX_ADDR_P2, address[0]);
      } else if(pipe == 3){
        RF_Register_Write(RF_REGISTER_RX_ADDR_P3, address[0]);
      } else if(pipe == 4){
        RF_Register_Write(RF_REGISTER_RX_ADDR_P4, address[0]);
      } else if(pipe == 5){
        RF_Register_Write(RF_REGISTER_RX_ADDR_P5, address[0]);
      }
      RF_Register_Write(RF_REGISTER_RX_PW_P0, RF_PAYLOAD_SIZE);
      byte en_rx_addr_read = RF_Register_Read(RF_REGISTER_EN_RXADDR);
      RF_Register_Write(RF_REGISTER_EN_RXADDR, (en_rx_addr_read | pipe_write));
    }
  } else {
    byte en_rx_addr_read = RF_Register_Read(RF_REGISTER_EN_RXADDR);
    RF_Register_Write(RF_REGISTER_EN_RXADDR, (en_rx_addr_read & (~pipe_write)));
  }
}

void Radio::RF_Set_Writing_Pipe(byte address[5]){
  RF_Register_Write_Mult(RF_REGISTER_RX_ADDR_P0, address);
  RF_Register_Write_Mult(RF_REGISTER_TX_ADDR, address);
  RF_Register_Write(RF_REGISTER_RX_PW_P0, RF_PAYLOAD_SIZE); 
}

void Radio::RF_Set_Power_Level(int level){
  byte setup_read = RF_Register_Read(RF_REGISTER_RF_SETUP);
  if(level == 0){
    RF_Register_Write(RF_REGISTER_RF_SETUP, ((setup_read & 0xF9) | 0x06));
  } else if(level == -6) {
    RF_Register_Write(RF_REGISTER_RF_SETUP, ((setup_read & 0xF9) | 0x04));
  } else if(level == -12) {
    RF_Register_Write(RF_REGISTER_RF_SETUP, ((setup_read & 0xF9) | 0x02));
  } else if(level == -18) {
    RF_Register_Write(RF_REGISTER_RF_SETUP, (setup_read & 0xF9));
  }
}

void Radio::RF_Listen(bool enable) {
  if(enable == true){
    RF_Register_Write(RF_REGISTER_CONFIG, (RF_Register_Read(RF_REGISTER_CONFIG) | 0x01));
    RF_Register_Write(RF_REGISTER_STATUS, (RF_Register_Read(RF_REGISTER_STATUS) | 0x70));
    digitalWrite (RF_CE, HIGH);
    if(RF_Register_Read(RF_REGISTER_FEATURE) & 0x02){
      RF_Flush_Tx();
    }
  } else {
    digitalWrite (RF_CE, LOW);
    delay(100);
    if(RF_Register_Read(RF_REGISTER_FEATURE) & 0x02){
      delayMicroseconds(200);
      RF_Flush_Tx();
    }
    RF_Register_Write(RF_REGISTER_CONFIG, (RF_Register_Read(RF_REGISTER_CONFIG) & 0xFE));
    RF_Register_Write(RF_REGISTER_EN_RXADDR, (RF_Register_Read(RF_REGISTER_EN_RXADDR) | 0x01));
  }
}

//***********
//  BUFFERS
//***********
void Radio::RF_Flush_Tx() {
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_FLUSH_TX);
  digitalWrite (RF_CSN, HIGH);
}

void Radio::RF_Flush_Rx() {
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_FLUSH_RX);
  digitalWrite (RF_CSN, HIGH);
}

//****************
//  TRANSMISSION
//****************

RF_data Radio::RF_Recieve() {
  char message[32] = "";
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_R_RX_PAYLOAD);
  for(int i = 0; i < 32; i++){
    message[i] = SPI.transfer(0xFF);
  }
  digitalWrite (RF_CSN, HIGH);
  RF_Register_Write(RF_REGISTER_STATUS, 0x70);

  return {message[0], message[1], message[2], message[3], message[4]}; 
}

bool Radio::RF_Send(RF_data data) {
  int no_ack = 0;
  byte message[5] = {data.thrust, data.x, data.y, data.rot, data.data};
  
  digitalWrite (RF_CSN, LOW);
  SPI.transfer(RF_WORD_W_TX_PAYLOAD);
  for(int i = 0; i < 5; i++){
    SPI.transfer(message[i]);
  }
  for(int i = 0; i < 27; i++){
    SPI.transfer(0x00);
  }
  digitalWrite (RF_CSN, HIGH);
  digitalWrite (RF_CE, HIGH);
  while(!(RF_Get_Status() & 0x30));
  digitalWrite (RF_CE, LOW);
  RF_Register_Write(RF_REGISTER_STATUS, 0x70);
  /*if(RF_Get_Status() & 0x10){
      Serial.println("WARNING: NO ACK");
      RF_Flush_Tx();
      return false;
  }*/
  return true;
}

//********
//  MISC
//********
void Radio::RF_Power_Enable(bool enable) {
  if(enable == true){
    int config_read = RF_Register_Read(RF_REGISTER_CONFIG);
    RF_Register_Write(RF_REGISTER_CONFIG, (config_read | 0x02));
  } else {
    int config_read = RF_Register_Read(RF_REGISTER_CONFIG);
    RF_Register_Write(RF_REGISTER_CONFIG, (config_read & 0xFD));
  }
}

bool Radio::RF_Message_Available(){
  if (!(RF_Register_Read(RF_REGISTER_FIFO_STATUS) & 0x01)){
    return true;
  }
  return false;
}

byte Radio::RF_Get_Status(){
  byte status_read;
  digitalWrite (RF_CSN, LOW);
  status_read = SPI.transfer(RF_WORD_W_TX_PAYLOAD);
  digitalWrite (RF_CSN, HIGH);
  return status_read;
}

//***************
//  DIAGNOSTICS
//***************

void Radio::DIAG_Print_All_Registers(){
  Serial.println("PRINTING ALL REGISTERS");
  for(byte i = 0; i < 32; i++){
    if(i == RF_REGISTER_RX_ADDR_P0 || i == RF_REGISTER_RX_ADDR_P1){ //Both registers are 5-bits long
      digitalWrite (RF_CSN, LOW);
      SPI.transfer(RF_WORD_R_REGISTER | (RF_MASK_READ & i));
      byte data_0 = SPI.transfer(0x00);
      byte data_1 = SPI.transfer(0x00);
      byte data_2 = SPI.transfer(0x00);
      byte data_3 = SPI.transfer(0x00);
      byte data_4 = SPI.transfer(0x00);
      digitalWrite (RF_CSN, HIGH);
      
      Serial.print(i, HEX);
      Serial.print("  ");
      Serial.print(data_0, HEX);
      Serial.print(" ");
      Serial.print(data_1, HEX);
      Serial.print(" ");
      Serial.print(data_2, HEX);
      Serial.print(" ");
      Serial.print(data_3, HEX);
      Serial.print(" ");
      Serial.print(data_4, HEX);
      Serial.print("  ");
      Serial.print(data_0, BIN);
      Serial.print("  ");
      Serial.print(data_1, BIN);
      Serial.print("  ");
      Serial.print(data_2, BIN);
      Serial.print("  ");
      Serial.print(data_3, BIN);
      Serial.print("  ");
      Serial.println(data_4, BIN);
    } else {
      int data = RF_Register_Read(i);
      Serial.print(i, HEX);
      Serial.print("  ");
      Serial.print(data, HEX);
      Serial.print("  ");
      Serial.println(data, BIN);
    }
  }
}

void Radio::DIAG_Print_Register(byte reg){
  Serial.print("PRINTING ");
  Serial.print(reg,HEX);
  Serial.println(" REGISTER");
  int data = RF_Register_Read(reg);
  Serial.print(reg, HEX);
  Serial.print("  ");
  Serial.println(data, BIN);
}
