#ifndef RADIO
#define RADIO

#include <SPI.h>

//RF REGISTER STATUS
#define RF_REGISTER_CONFIG        0x00
#define RF_REGISTER_EN_AA         0x01
#define RF_REGISTER_EN_RXADDR     0x02
#define RF_REGISTER_SETUP_AW      0x03
#define RF_REGISTER_SETUP_RETR    0x04
#define RF_REGISTER_RF_CH         0x05
#define RF_REGISTER_RF_SETUP      0x06
#define RF_REGISTER_STATUS        0x07
#define RF_REGISTER_OBSERVE_TX    0x08
#define RF_REGISTER_RPD           0x09
#define RF_REGISTER_RX_ADDR_P0    0x0A
#define RF_REGISTER_RX_ADDR_P1    0x0B
#define RF_REGISTER_RX_ADDR_P2    0x0C
#define RF_REGISTER_RX_ADDR_P3    0x0D
#define RF_REGISTER_RX_ADDR_P4    0x0E
#define RF_REGISTER_RX_ADDR_P5    0x0F
#define RF_REGISTER_TX_ADDR       0x10
#define RF_REGISTER_RX_PW_P0      0x11
#define RF_REGISTER_RX_PW_P1      0x12
#define RF_REGISTER_RX_PW_P2      0x13
#define RF_REGISTER_RX_PW_P3      0x14
#define RF_REGISTER_RX_PW_P4      0x15
#define RF_REGISTER_RX_PW_P5      0x16
#define RF_REGISTER_FIFO_STATUS   0x17
#define RF_REGISTER_DYNPD         0x1C
#define RF_REGISTER_FEATURE       0x1D

//RF WORD
#define RF_WORD_R_REGISTER        0x00
#define RF_WORD_W_REGISTER        0x20
#define RF_WORD_R_RX_PAYLOAD      0x61
#define RF_WORD_W_TX_PAYLOAD      0xA0
#define RF_WORD_FLUSH_TX          0xE1
#define RF_WORD_FLUSH_RX          0xE2
#define RF_WORD_REUSE_TX_PL       0xE3
#define RF_WORD_R_RX_PL_WID       0x60
//#define RF_WORD_W_ACK_PAYLOAD  
#define RF_WORD_W_TX_PAYLOAD_NO   0xB0
#define RF_WORD_NOP               0xFF

//MASKS
#define RF_MASK_READ              0x1F
#define RF_MASK_WRITE             0x1F

//GLOBAL DEFINES
#define RF_PAYLOAD_SIZE           0x20

//PINS
#define RF_CE                     7
#define RF_CSN                    8

#ifndef STRUCTS
#define STRUCTS

struct Axes {
    float x;
    float y;
    float z;
};

struct Axes_long {
    long x;
    long y;
    long z;
};

struct RF_data {
  byte thrust;
  byte x;
  byte y;
  byte rot;
  byte data;
};

#endif 

class Radio {
    public:
    void RF_init();
    void RF_Set_Reading_Pipe(int pipe, bool enable, byte address[5]);
    void RF_Set_Writing_Pipe(byte address[5]);
    void RF_Set_Power_Level(int level);
    void RF_Listen(bool enable);
    RF_data RF_Recieve();
    bool RF_Send(RF_data data);
    bool RF_Message_Available();
    void RF_Flush_Rx();

    private:
    void RF_Register_Write(byte reg, byte value);
    void RF_Register_Write_Mult(byte reg, byte value[5]);
    byte RF_Register_Read(byte reg);
    void RF_Set_Data_Rate(int rate);
    void RF_Set_Channel(byte channel);
    void RF_Flush_Tx();
    //void RF_Flush_Rx();
    void RF_Power_Enable(bool enable);
    byte RF_Get_Status();
    void DIAG_Print_All_Registers();
    void DIAG_Print_Register(byte reg);
};

#endif
