/*  DRONE CODE
 *  AUTHOR: HENRY W TRIFF
 * 
 *  NOTES TO SELF:
 *  > nan Error with gyro when dramatic change in speed. Potentially to do with using byte instead of int8_t or something
 */

//LIBRARIES
#include <SPI.h>
#include <Wire.h>
#include "Radio.h"
#include "Gyro.h"
#include "Mag.h"
#include "Motors.h"
#include "PID.h"

//OBJECTS
Radio Radio;
Gyro Gyro;
Mag Mag;
Motors Motors;
PID PID;

//DEFINITIONS
#define MOTOR_NORTH_OFFSET 16
#define MOTOR_SOUTH_OFFSET 17
#define MOTOR_EAST_OFFSET 15
#define MOTOR_WEST_OFFSET 14

#ifndef STRUCTS
#define STRUCTS

struct Axes {
    float x;
    float y;
    float z;
};

struct Axes_long {
    long x;
    long y;
    long z;
};

struct RF_data {
  byte thrust;
  byte x;
  byte y;
  byte rot;
  byte data;
};

#endif 

//GLOBAL VARIABLES
byte address[6] = "Drone";
unsigned int thrust = 0;
unsigned int proportional_data = 0;
unsigned int differential_data = 0;
unsigned int integral_data = 0;
float proportional = 0;
float differential = 0;
float integral = 0;
int no_signal = 0;

void setup() {
  Serial.begin(115200);
  Wire.begin();

  //Mag.MG_init();
  //Mag.MG_Callibrate();
  Gyro.GY_init();

  Radio.RF_init();
  Radio.RF_Set_Reading_Pipe(0, true, address);
  Radio.RF_Set_Power_Level(-18);
  Radio.RF_Listen(true);
  
  Motors.MO_init();

  Serial.println("Start");

}

void loop() {

  if(Radio.RF_Message_Available() == true){
    no_signal = 0;
    RF_data data = Radio.RF_Recieve();
    thrust = data.thrust;
    proportional_data = data.x;
    differential_data = data.y;
    integral_data = data.rot;
    if(thrust > 100) {
      thrust = 100;
    }

    proportional_data = constrain(proportional_data, 0,200);
    proportional = (float)proportional_data / 10;
    differential_data = constrain(differential_data, 0,200);
    differential = (float)differential_data / 100;
    integral_data = constrain(integral_data, 0,100);
    integral = (float)integral_data / 1000;
    
    Radio.RF_Flush_Rx();
  } else {
    no_signal++;
    if(no_signal > 200){
      Serial.println("ERROR: No Signal");
      Motors.MO_Set(0,0,0,0);
      while(1);
    }
  }
  
  Gyro.GY_Get_Axes();

  Axes gyro_angle;
  Axes gyro_rate_angle;
  Axes desire = {0,0,0};
  
  gyro_angle = Gyro.Gyro_value;
  gyro_rate_angle.x = Gyro.Gyro_Raw.x;
  gyro_rate_angle.y = Gyro.Gyro_Raw.y; 

  float angle_magnitude = sqrt(gyro_angle.x * gyro_angle.x + gyro_angle.y * gyro_angle.y);
  
  if(angle_magnitude > 60 || angle_magnitude < -60){
    Motors.MO_Set(0,0,0,0);
    Serial.println("ERROR: Drone Overturned");
    while(1);
  }
  
  int north_power = 0;
  int south_power = 0;
  int east_power = 0;
  int west_power = 0;

  if(thrust >= 1){
    
    Axes PID_output = PID.Conversion(gyro_angle, desire, proportional, differential, integral);

    north_power = thrust*10 + PID_output.x + MOTOR_NORTH_OFFSET;
    south_power = thrust*10 - PID_output.x + MOTOR_SOUTH_OFFSET;
    east_power = thrust*10 + PID_output.y +  MOTOR_EAST_OFFSET;
    west_power = thrust*10 - PID_output.y + MOTOR_WEST_OFFSET;

  }

  north_power = constrain(north_power,0,500);
  south_power = constrain(south_power,0,500);
  east_power = constrain(east_power,0,500);
  west_power = constrain(west_power,0,500);
  
  Motors.MO_Set(north_power,south_power,east_power,west_power);

}
